% =========================================================================
% Code for "Pandemic Recessions and Contact Tracing"
% by Leonardo Melosi and Matthias Rottner
% Journal of the European Economic Association, forthcoming
% This file computes Table 2 and Figures 2-4 of the paper. 
% It also creates Figures B1, C1, G1, G2, H1, I1, I2 of the appendix. 
% =========================================================================

% This is the main file and produces all results.
% It calls all other functions. The structure of this file is as follows:
%   -- Optimal lockdown policy is calculated in the case of no testing
%   -- Optimal lockdown policy is calculated for the case of tracing
%   -- Dynamics of different economies considered in Table 2 are calculated
%   -- Table 2 is printed
%   -- Dynamics of additional scenarios for the figures are calculated
%   -- Figures 2-4 are created
%   -- Figures of the Online Appendix are created

%% Start
close all; clear all; clc;

tic
addpath Files  % links to files that are called within this m file
addpath ExternalFiles % links to external files that are called within this file 

%% Optimal Policy No Tracing and Testing
get_setup
T = 250;
P.tol = 0.001; 0.00001;
pf_n_s_guess = P.N*ones(T,1);  % Labor of susceptible
pf_n_i_guess = P.N*ones(T,1);  % Labor of infected     
pf_n_r_guess = P.N*ones(T,1);  % Labor of recovered 
muc_guess = 0;

clear getU_err
HH_final = 150;
muc_save = zeros(HH_final,HH_final);

% Note that the calculation can take quite some time, especially without
% several cores. Therefore, we have chosen as default that this optimization
% is skipped and the optimal tax schedule are read in directly
opt_cal1 = 0; % Set to 1 if optimization should be done by the computer

if opt_cal1 == 1

    for HH = HH_final
        opts_fmincon=optimoptions('fmincon','Display','iter','TolFun',1e-9,'MaxFunctionEvaluations',20000,'FiniteDifferenceStepSize',1e-3,'UseParallel',true); %options for fmincon w/o parallel comp.

        muc_guess = 0.05*ones(HH,1);

         LB=muc_guess*0;-2; UB=muc_guess+10;%lower and upper bounds
        muc_sol = fmincon(@getU_err,muc_guess,[],[],[],[],LB,UB,[],opts_fmincon,pf_n_s_guess,pf_n_i_guess,pf_n_r_guess,P,T);

        muc_save(1:length(muc_sol),HH) = muc_sol;
    end

    fend = getU_err(muc_sol,pf_n_s_guess,pf_n_i_guess,pf_n_r_guess,P,T);

    if HH == 150 && min(LB) == 0
        muc_sol_NoTrace_Bounded_HH150 = muc_sol;
        save Simulations/muc_sol_NoTrace_Bounded_HH150 muc_sol_NoTrace_Bounded_HH150 % Only Solution    
    end

end
%% Optimal policy - Contract Tracing

% Note that the calculation can take quite some time, especially without
% several cores. Therefore, we have chosen as default that this optimization
% is skipped and the optimal tax schedule are read in directly
opt_cal2 = 0; % Set to 1 if optimization should be done by the computer

if opt_cal2 == 1
    
    scenarios_HH = [150 150 30 30];
    scenarios_choice = [1 2 1 2];

    for ll = 1:4

        get_setup;
        HH_final = scenarios_HH(ll); 250; 150; 30; 250;
        P.Choice_Ups = scenarios_choice(ll); 
        P.scale_opt= 10;
        P.trace = 1; 
        T = 250;
        P.tol = 0.0001; 
        pf_n_s_guess = P.N*ones(T,1);  % Labor of susceptible
        pf_n_i_guess = P.N*ones(T,1);  % Labor of infected     
        pf_n_r_guess = P.N*ones(T,1);  % Labor of recovered 
        pf_n_c_guess = P.N*ones(T,1);  % Labor of recovered 

        muc_save_trace = zeros(HH_final,HH_final);

        P.binopdf = 2;  % Choose your approximation
        P.k_s_max = 5;
        P.grid = 0:P.k_s_max;

        clear getU_trace_alphac_err

        for HH = HH_final
            opts_fmincon=optimoptions('fmincon','Display','iter','TolFun',1e-9,'MaxFunctionEvaluations',20000,'FiniteDifferenceStepSize',1e-5,'UseParallel',true); %options for fmincon w/o parallel comp.

            muc_guess = P.scale_opt*0.5*ones(HH,1);% muc_sol;   

            LB=P.scale_opt*muc_guess*0;UB=muc_guess+10*P.scale_opt;%lower and upper bounds
            muc_sol = fmincon(@getU_trace_alphac_err,muc_guess,[],[],[],[],LB,UB,[],opts_fmincon,pf_n_s_guess,pf_n_i_guess,pf_n_r_guess,pf_n_c_guess,P,T);
            muc_save_trace(1:length(muc_sol),HH) = muc_sol;
            muc_sol = muc_sol./P.scale_opt;

        end
        
        fend_trace = getU_trace_alphac_err(muc_sol*P.scale_opt,pf_n_s_guess,pf_n_i_guess,pf_n_r_guess,pf_n_c_guess,P,T);

        % Save the results
        if HH == 150
            if P.Choice_Ups == 2
                muc_sol_trace_unbounded_HH150 = muc_sol;
                save Simulations/muc_sol_trace_unbounded_HH150 muc_sol_trace_unbounded_HH150
            elseif  P.Choice_Ups == 1
                muc_sol_trace_unlimitedtests_HH150= muc_sol;
                save Simulations/muc_sol_trace_unlimitedtests_HH150 muc_sol_trace_unlimitedtests_HH150
            end    
        elseif HH == 30
            if P.Choice_Ups == 2
                muc_sol_trace_unbounded_HH30 = muc_sol;
                save Simulations/muc_sol_trace_unbounded_HH30 muc_sol_trace_unbounded_HH30
            elseif  P.Choice_Ups == 1
                muc_sol_trace_unlimitedtests_HH30= muc_sol;
                save Simulations/muc_sol_trace_unlimitedtests_HH30 muc_sol_trace_unlimitedtests_HH30
            end
        end
    end
end



%% Calculations Table 2

% Choose different setups

trace_vec = [1 1 1 8 1 2 7 0 0 11];
ups_vec =   [2 2 2 2 1 2 1 1 1 1];
lock_vec =  [1 7 7 1 1 1 1 1 7 1];
lenlo_vec = [0 1 2 0 0 0 0 0 2 0];

get_setup;

pii_mat = zeros(4,length(trace_vec));

disp('To be calculated ...')
for m = 1:length(trace_vec)
    disp(['Table Scenario: ' num2str(m) '/' num2str(length(trace_vec))])
    P.trace = trace_vec(m);  % 7: Tracing + Quarantine; 8: Tracing + Testing + Quarantine; 1: Basic tracing + testing; 5: Combine basic tracing with random testing
    P.Choice_Ups = ups_vec(m); % Limites tests, but does not affect capacity of quarantine; 1: unlimited tests
    P.Choice_lock = lock_vec(m); 
    
    if P.trace==1 &&  P.Choice_Ups == 2 && P.Choice_lock == 7 && lenlo_vec(m) == 2% Limited Tests + Long Lockdown
        load Simulations/muc_sol_trace_unbounded_HH150 muc_sol_trace_unbounded_HH150
        P.muc_sol_in = muc_sol_trace_unbounded_HH150;
    elseif P.trace==1 &&  P.Choice_Ups == 2 && P.Choice_lock == 7 && lenlo_vec(m) == 1% Limited Tests + Short Lockdown
        load Simulations/muc_sol_trace_unbounded_HH30 muc_sol_trace_unbounded_HH30
        P.muc_sol_in = muc_sol_trace_unbounded_HH30;
    elseif P.trace==1 &&  P.Choice_Ups == 1 && P.Choice_lock == 7  && lenlo_vec(m) == 2  % Unlimites Tests + Long Lockdown
        load Simulations/muc_sol_trace_unlimitedtests_HH150 muc_sol_trace_unlimitedtests_HH150
         P.muc_sol_in = muc_sol_trace_unlimitedtests_HH150;
     elseif P.trace==1 &&  P.Choice_Ups == 1 && P.Choice_lock == 7  && lenlo_vec(m) == 1  % Unlimites Tests + Short Lockdown
        load Simulations/muc_sol_trace_unlimitedtests_HH30 muc_sol_trace_unlimitedtests_HH30
         P.muc_sol_in = muc_sol_trace_unlimitedtests_HH30;
    elseif P.trace==0 &&  P.Choice_Ups == 1 && P.Choice_lock == 7  && lenlo_vec(m) == 2  % No testing + Long Lockdown
        load Simulations/muc_sol_NoTrace_Bounded_HH150 muc_sol_NoTrace_Bounded_HH150 
        P.muc_sol_in = muc_sol_NoTrace_Bounded_HH150;
    end
    [pf] = eqm_Exposure(P,T); 
    
    eval(['pf_' num2str(P.trace) num2str(P.Choice_Ups) num2str(P.Choice_lock) '= pf;'])
    eval(['save ' opts.saveSimulations 'pf_' num2str(P.trace) num2str(P.Choice_Ups) num2str(P.Choice_lock) ' pf_' num2str(P.trace) num2str(P.Choice_Ups) num2str(P.Choice_lock)])
    if P.trace==1 &&  P.Choice_Ups == 2 && P.Choice_lock == 7 && lenlo_vec(m) == 1%
        eval(['pf_30_' num2str(P.trace) num2str(P.Choice_Ups) num2str(P.Choice_lock) '= pf;'])
        eval(['save ' opts.saveSimulations 'pf_30_' num2str(P.trace) num2str(P.Choice_Ups) num2str(P.Choice_lock) ' pf_30_' num2str(P.trace) num2str(P.Choice_Ups) num2str(P.Choice_lock)])
    end
    pii_mat(:,m)= [pf.Util_s(1),mean(pf.Cons),pf.D(end-1),pf.R(end-1)];
end



% Consumption Equivalent
pii_mat(5,:) = 100*(exp((1-P.beta)*(pii_mat(1,:)-P.Util))-1);
% Money Losses per agent in thousand dollars
pii_mat(6,:) = (pii_mat(5,:)/100)*P.C/(1-P.beta)/1000;
% Economic Cost in trillion dollars
pii_mat(7,:) = pii_mat(6,:)*1e3*(328*1e6)/(1e+12);
% Consumption Loss
pii_mat(8,:) = (pii_mat(2,:)-P.C)/P.C*100;


%%
disp(' ')
disp(' ')
disp('==========================================================================')
disp('                              Table 2                                     ')
disp('--------------------------------------------------------------------------')
disp('                                          Wel     Cons   Mor    Rec  Costs')
disp('--------------------------------------------------------------------------')
disp('                 Contact tracing with limited testing                     ')
disp('--------------------------------------------------------------------------')
disp(['No social distancing:                    ' num2str(round(pii_mat(5,1),2)) '   '  num2str(round(pii_mat(8,1),2)) '   '  num2str(round(pii_mat(3,1)*100,2)) '    ' num2str(round(pii_mat(4,1)*100,0)) '   '  num2str(round(-pii_mat(7,1),2))])
disp(['Optimal social distancing (short):       ' num2str(round(pii_mat(5,2),2)) '   '  num2str(round(pii_mat(8,2),2)) '   '  num2str(round(pii_mat(3,2)*100,2)) '   ' num2str(round(pii_mat(4,2)*100,0)) '   '  num2str(round(-pii_mat(7,2),2))])
disp(['Optimal social distancing (long):        ' num2str(round(pii_mat(5,3),2)) '   '  num2str(round(pii_mat(8,3),2)) '   '  num2str(round(pii_mat(3,3)*100,2)) '   ' num2str(round(pii_mat(4,3)*100,0)) '   '  num2str(round(-pii_mat(7,3),2))])
disp(['Tighter quarantine:                      ' num2str(round(pii_mat(5,4),2)) '   '  num2str(round(pii_mat(8,4),2)) '   '  num2str(round(pii_mat(3,4)*100,2)) '   ' num2str(round(pii_mat(4,4)*100,0)) '   '  num2str(round(-pii_mat(7,4),2))])
disp('--------------------------------------------------------------------------')
disp('                 Alternative contact tracing scenarios                    ')
disp('--------------------------------------------------------------------------')
disp(['Unlimited testing capacity:              ' num2str(round(pii_mat(5,5),2)) '   '  num2str(round(pii_mat(8,5),2)) '   '  num2str(round(pii_mat(3,5)*100,2)) '   ' num2str(round(pii_mat(4,5)*100,0)) '   '  num2str(round(-pii_mat(7,5),2))])
disp(['Comprehensive tracing:                   ' num2str(round(pii_mat(5,6),2)) '   '  num2str(round(pii_mat(8,6),2)) '   '  num2str(round(pii_mat(3,6)*100,2)) '   ' num2str(round(pii_mat(4,6)*100,0)) '   '  num2str(round(-pii_mat(7,6),2))])
disp(['Unlimited testing capacity:              ' num2str(round(pii_mat(5,7),2)) '    '  num2str(round(pii_mat(8,7),2)) '   '  num2str(round(pii_mat(3,7)*100,2)) '   ' num2str(round(pii_mat(4,7)*100,0)) '   '  num2str(round(-pii_mat(7,7),2))])
disp('--------------------------------------------------------------------------')
disp('                          No contact tracing                              ')
disp('--------------------------------------------------------------------------')
disp(['No testing:                              ' num2str(round(pii_mat(5,8),2)) '   '  num2str(round(pii_mat(8,8),2)) '   '  num2str(round(pii_mat(3,8)*100,2)) '   ' num2str(round(pii_mat(4,8)*100,0)) '   '  num2str(round(-pii_mat(7,8),2))])
disp(['No testing + optimal social distancing:  ' num2str(round(pii_mat(5,9),2)) '   '  num2str(round(pii_mat(8,9),2)) '   '  num2str(round(pii_mat(3,9)*100,2)) '   ' num2str(round(pii_mat(4,9)*100,0)) '   '  num2str(round(-pii_mat(7,9),2))])
disp(['Random testing:                          ' num2str(round(pii_mat(5,10),2)) '    '  num2str(round(pii_mat(8,10),2)) '   '  num2str(round(pii_mat(3,10)*100,2)) '   ' num2str(round(pii_mat(4,10)*100,0)) '   '  num2str(round(-pii_mat(7,10),2))])
disp('==========================================================================')
disp(' ')
disp(' ')


%%

%% Run Some Additional Setups for the Figures (that have not been needed for the Table 2)

get_setup
trace_vec = [7 1 5 2];
ups_vec =   [2 2 2 1];
lock_vec =  [1 1 1 1];


disp('To be calculated')
for m = 1:length(trace_vec)
    disp(['Different Models: ' num2str(m) '/' num2str(length(trace_vec))])
    P.trace = trace_vec(m);  % 7: Tracing + Quarantine; 8: Tracing + Testing + Quarantine; 1: Basic tracing + testing; 5: Combine basic tracing with random testing
    P.Choice_Ups = ups_vec(m); % 2: Limites tests, but does not affect capacity of quarantine; 1: unlimited tests
    P.Choice_lock = lock_vec(m); % 1: no lockdown
    [pf] = eqm_Exposure(P,T); 
    eval(['pf_' num2str(P.trace) num2str(P.Choice_Ups) num2str(P.Choice_lock) '= pf;'])
    eval(['save ' opts.saveSimulations 'pf_' num2str(P.trace) num2str(P.Choice_Ups) num2str(P.Choice_lock) ' pf_' num2str(P.trace) num2str(P.Choice_Ups) num2str(P.Choice_lock)])
    
end
%% Figure 2 - 4

Figure2_main_run
Figure3_main_run
Figure4_main_run

%% Online Appendix: Figure B1, C1, G1, G2, H1, I1, I2

FigureB1_main_run
FigureC1_main_run
FigureG1_main_run
FigureG2_main_run  % contains choice between loading results or generating results due to computational time
FigureH1_main_run  % contains choice between loading results or generating results due to computational time
FigureI1_main_run
FigureI2_main_run

toc


